Package: aion
Title: Archaeological Time Series
Version: 1.7.1
Authors@R: c(
    person("Nicolas", "Frerebeau", , "nicolas.frerebeau@u-bordeaux-montaigne.fr", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0001-5759-4944")),
    person("Joe", "Roe", , "joseph.roe@iaw.unibe.ch", role = "aut",
           comment = c(ORCID = "0000-0002-1011-1244")),
    person("Brice", "Lebrun", role = "art",
           comment = c(ORCID = "0000-0001-7503-8685", "Logo designer")),
    person("Université Bordeaux Montaigne", role = "fnd",
           comment = c(ROR = "03pbgwk21")),
    person("CNRS", role = "fnd",
           comment = c(ROR = "02feahw73"))
  )
Maintainer: Nicolas Frerebeau <nicolas.frerebeau@u-bordeaux-montaigne.fr>
Description: A toolkit for archaeological time series and time intervals.
    This package provides a system of classes and methods to represent and
    work with archaeological time series and time intervals. Dates are
    represented as "rata die" and can be converted to (virtually) any
    calendar defined by Reingold and Dershowitz (2018)
    <doi:10.1017/9781107415058>. This packages offers a simple API that
    can be used by other specialized packages.
License: GPL (>= 3)
URL: https://codeberg.org/tesselle/aion,
        https://packages.tesselle.org/aion/,
        https://tesselle.r-universe.dev/aion
BugReports: https://codeberg.org/tesselle/aion/issues
Depends: R (>= 3.5)
Imports: arkhe (>= 1.10.0), graphics, grDevices, methods, stats, utils
Suggests: folio (>= 1.5.0), fontquiver, igraph, knitr, markdown,
        relations, rsvg, svglite, tinysnapshot (>= 0.2.0), tinytest
VignetteBuilder: knitr
Encoding: UTF-8
LazyData: true
X-schema.org-applicationCategory: Archaeological Science
X-schema.org-isPartOf: https://www.tesselle.org
X-schema.org-keywords: time-series, time-intervals, chronology,
        stratigraphy, archaeology, archaeological-science, r-package
Collate: 'AllClasses.R' 'AllGenerics.R' 'aion-internal.R'
        'aion-package.R' 'axis.R' 'calendar-gregorian.R'
        'calendar-julian.R' 'calendar.R' 'coerce.R' 'convert.R'
        'data.R' 'format.R' 'graph.R' 'intervals.R' 'mutators.R'
        'operators.R' 'overlap.R' 'plot.R' 'relations.R' 'series.R'
        'show.R' 'span.R' 'subset.R' 'time.R' 'validate.R' 'year.R'
        'zzz.R'
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-06-23 09:43:33 UTC; nfrerebeau
Author: Nicolas Frerebeau [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-5759-4944>),
  Joe Roe [aut] (ORCID: <https://orcid.org/0000-0002-1011-1244>),
  Brice Lebrun [art] (ORCID: <https://orcid.org/0000-0001-7503-8685>,
    Logo designer),
  Université Bordeaux Montaigne [fnd] (ROR: <https://ror.org/03pbgwk21>),
  CNRS [fnd] (ROR: <https://ror.org/02feahw73>)
Repository: CRAN
Date/Publication: 2026-06-23 10:20:02 UTC
Built: R 4.6.0; ; 2026-06-23 11:58:16 UTC; unix
