# Generated by roxygen2: do not edit by hand

S3method(`names<-`,TimeIntervals)
S3method(as.array,TimeSeries)
S3method(as.data.frame,TimeIntervals)
S3method(as.data.frame,TimeSeries)
S3method(end,TimeIntervals)
S3method(end,TimeSeries)
S3method(format,RataDie)
S3method(format,TimeIntervals)
S3method(format,TimeScale)
S3method(frequency,TimeSeries)
S3method(image,TimeSeries)
S3method(labels,TimeIntervals)
S3method(labels,TimeSeries)
S3method(length,TimeIntervals)
S3method(names,TimeIntervals)
S3method(plot,TimeIntervals)
S3method(plot,TimeSeries)
S3method(pretty,RataDie)
S3method(start,TimeIntervals)
S3method(start,TimeSeries)
S3method(time,TimeSeries)
S3method(window,TimeSeries)
export(AD)
export(BC)
export(BCE)
export(BP)
export(CE)
export(J)
export(b2k)
export(fixed_from_AD)
export(fixed_from_BC)
export(fixed_from_BCE)
export(fixed_from_BP)
export(fixed_from_CE)
export(fixed_from_b2k)
export(fixed_from_julian)
export(fixed_to_AD)
export(fixed_to_BC)
export(fixed_to_BCE)
export(fixed_to_BP)
export(fixed_to_CE)
export(fixed_to_b2k)
export(fixed_to_julian)
export(get_calendar)
export(is_calendar)
export(is_gregorian)
export(is_julian)
export(set_calendar)
export(year_axis)
exportClasses(RataDie)
exportClasses(TimeIntervals)
exportClasses(TimeScale)
exportClasses(TimeSeries)
exportMethods("[")
exportMethods(Arith)
exportMethods(as.array)
exportMethods(as.data.frame)
exportMethods(as_date)
exportMethods(as_decimal)
exportMethods(as_fixed)
exportMethods(as_year)
exportMethods(calendar)
exportMethods(calendar_direction)
exportMethods(calendar_epoch)
exportMethods(calendar_fixed)
exportMethods(calendar_label)
exportMethods(calendar_name)
exportMethods(calendar_unit)
exportMethods(calendar_year)
exportMethods(contains)
exportMethods(convert)
exportMethods(during)
exportMethods(end)
exportMethods(equals)
exportMethods(finished_by)
exportMethods(finishes)
exportMethods(fixed)
exportMethods(flip)
exportMethods(format)
exportMethods(frequency)
exportMethods(graph_create)
exportMethods(graph_prune)
exportMethods(image)
exportMethods(intervals)
exportMethods(meets)
exportMethods(met_by)
exportMethods(overlap)
exportMethods(overlapped_by)
exportMethods(overlaps)
exportMethods(plot)
exportMethods(preceded_by)
exportMethods(precedes)
exportMethods(pretty)
exportMethods(series)
exportMethods(span)
exportMethods(start)
exportMethods(started_by)
exportMethods(starts)
exportMethods(time)
exportMethods(window)
import(arkhe)
import(methods)
importFrom(stats,end)
importFrom(stats,frequency)
importFrom(stats,start)
importFrom(stats,time)
importFrom(stats,window)
